/*
 * java.lang.StringBuffer: part of the Java Class Libraries project.
 * Copyright (C) 1998 Free Software Foundation
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */

package net.sourceforge.jedit.quickbuffer;

import javax.swing.*;
import javax.swing.table.*; 
 
import org.gjt.sp.jedit.*;

import java.io.File;
import java.util.Vector;

public class BufferTable extends JTable {
    
    /**
    This is a kludge to get directory strings as "...ectory" instead of "direc.."
    */
    public final static int FONT_WIDTH = 8;


    private Buffer current = null;
    
    public BufferTable() {
        super( new BufferTableModel() );

        DefaultListSelectionModel selectionModel = 
            new DefaultListSelectionModel();
        selectionModel.setSelectionMode( DefaultListSelectionModel.SINGLE_SELECTION );
        
        this.setSelectionModel( selectionModel );
        
        this.setAutoResizeMode( JTable.AUTO_RESIZE_ALL_COLUMNS );

    }
    
    public Buffer getSelectedFile() {

        int index = this.getSelectedRow() - 1;

        //stupid swing bug... this might not exist on alternate platforms
        if (index < 0 ) {
            index = this.getRowCount() - 1;
        }


          
        return ((BufferTableModel)this.getModel()).getBuffer( index );

    }

    public Buffer getBuffer() {
        return this.current;
    }
    public void setBuffer(Buffer current) {
        this.current = current;
    }
    
    private void initColumnSizes() {

        this.getColumnModel().getColumn(0).setWidth( this.getSize().width / 2 );
        this.getColumnModel().getColumn(1).setWidth( this.getSize().width / 2 );

    }
    
    public void refresh() {

        BufferTableModel.setBuffer( this.getBuffer() );

        ((BufferTableModel)this.getModel()).setDataVector( BufferTableModel.getTableData( this.getColumnModel().getColumn(1).getWidth() ),
                                                           BufferTableModel.getColumnNames() );

        this.initColumnSizes();
        

        int index = BufferTableModel.getCurrentBufferIndex();

        this.setRowSelectionInterval( index, index );
                                                           
    }
    
}


class BufferTableModel extends DefaultTableModel {
    
    private static int bufferIndex = 0;
    private static Buffer current = null;
    
    public BufferTableModel() {
        super(new Vector(), BufferTableModel.getColumnNames());
    }

    
    public static Buffer getBuffer() {
        return current;
    }
    public static void setBuffer(Buffer current) {
        BufferTableModel.current = current;
    }

    public Class getColumnClass(int c) {
        return getValueAt(0, c).getClass();
    }

    /*
     * Don't need to implement this method unless your table's
     * editable.
     */
    public boolean isCellEditable(int row, int col) {
        return false;
    }

    
    public Buffer getBuffer(int index) {
     
        return ((BufferWrapper)this.getValueAt( index, 0 )).getBuffer();

    }

    public static int getCurrentBufferIndex() {
        return bufferIndex;
    }
    
    /**
    @author <A HREF="mailto:burton@relativity.yi.org">Kevin A. Burton</A>
    @version $Id: BufferTable.java,v 1.1.1.1 2000/01/24 07:03:49 burtonator Exp $
    @param max_width used to chop off character data from the begining of a string
    */
    public static Vector getTableData(final int max_width) {

        Vector rows = new Vector();


        Buffer[] buffers = jEdit.getBuffers();

        //qsort this array of buffers
        MiscUtilities.quicksort( buffers, new BufferComparator() );

        for (int i = 0; i < buffers.length; ++i) {
            
            if ( BufferTableModel.getBuffer().equals( buffers[i] ) ) {
                BufferTableModel.bufferIndex = i;
            }
            
            Vector row = new Vector();
            
            row.addElement( new BufferWrapper( buffers[i] ) );
 
            //now determine how to trim the directory desc.
            
            String parent = buffers[i].getFile().getParent();

            

            
            int begin = ( parent.length() * BufferTable.FONT_WIDTH )  - max_width;
            
            System.out.println("begin == " + begin);
            
            if ( begin > 0 ) {
                
                //the number of chars..
                begin = begin / BufferTable.FONT_WIDTH;
                
                System.out.println("begin == " + begin);                
                
                parent = "..." + parent.substring( begin, parent.length() );
                
            }


            
            row.addElement( parent );
    

        rows.addElement( row );
        }

        
        return rows;
    }

    
    
    
    /**
    @author <A HREF="mailto:burton@relativity.yi.org">Kevin A. Burton</A>
    @version $Id: BufferTable.java,v 1.1.1.1 2000/01/24 07:03:49 burtonator Exp $
    */
    public static Vector getColumnNames() {

        Vector names = new Vector();
        names.addElement("File Name");
        names.addElement("Directory");
        return names;
    }

}



class BufferWrapper {
    
    private Buffer buffer = null;

    public BufferWrapper( Buffer buffer ) {
        this.buffer = buffer;
    }
    
    public Buffer getBuffer() {
        return this.buffer;
    }
   
    public String toString() {
        return this.buffer.getName();
    }
   
}
