/*
 * java.lang.StringBuffer: part of the Java Class Libraries project.
 * Copyright (C) 1998 Free Software Foundation
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */
package net.sourceforge.jedit.quickbuffer;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
 
import org.gjt.sp.jedit.*;

/**
@author <A HREF="mailto:burton@relativity.yi.org">Kevin A. Burton</A>
@version $Id: QuickBuffer.java,v 1.3 2000/08/08 00:05:51 burtonator Exp $
*/
public class QuickBuffer extends JDialog implements ActionListener, KeyListener, MouseListener {
    
    public final static int WIDTH   = 400;
    public final static int HEIGHT  = 300;


    
    private View            view    = null;
    private JButton         ok      = new JButton( "OK" );
    private JButton         cancel  = new JButton( "Cancel" );
    private BufferTable     table   = null;
    private boolean         init    = false;
    private JScrollPane     spane   = null;

    /**
    Center the QuickBuffer at least once.
    */
    private boolean         centered = false;
    
    public QuickBuffer() {

        super();

        this.setTitle("Quick Buffer");

        this.getContentPane().setLayout( new BorderLayout() );
        
    
        this.table = new BufferTable();

        this.addKeyListener( this );
        this.table.addKeyListener( this );
        this.table.addMouseListener( this );
       

        this.spane = new JScrollPane( this.table );
        this.getContentPane().add( spane, BorderLayout.CENTER );
        this.getContentPane().add( getButtons(), BorderLayout.SOUTH );        
        
        this.setEnabled(true);        
        this.setSize( new Dimension( WIDTH, HEIGHT ) );


    }

    private JPanel getButtons() {
        
        JPanel panel = new JPanel();
        
        panel.setLayout( new FlowLayout() );

        this.ok.addActionListener( this );
        this.cancel.addActionListener( this );
        
        panel.add( this.ok );
        panel.add( this.cancel );

        return panel;
        
    }

    /**
    Used to perform initialzation that should not be done within the constructor
    
    @author <A HREF="mailto:burton@relativity.yi.org">Kevin A. Burton</A>
    @version $Id: QuickBuffer.java,v 1.3 2000/08/08 00:05:51 burtonator Exp $
    */
    public void init() {
        this.init = true;
        this.show();
        this.toFront();
    }
    
    /**
    Returns true if init has been run on this buffer.
    
    @author <A HREF="mailto:burton@relativity.yi.org">Kevin A. Burton</A>
    @version $Id: QuickBuffer.java,v 1.3 2000/08/08 00:05:51 burtonator Exp $
    */
    public boolean hasInit() {
        return this.init;
    }
    
    /**
    Set the view of this quick buffer so that everytime you launch it she runs
    under the correct view.

    @author <A HREF="mailto:burton@relativity.yi.org">Kevin A. Burton</A>
    @version $Id: QuickBuffer.java,v 1.3 2000/08/08 00:05:51 burtonator Exp $
    */
    public void setView(View view) {
        this.view = view;
        this.center();
    }

    public void setVisible( boolean visible ) {

        if ( visible == true ) {

            this.table.setBuffer( this.getView().getBuffer() );
            this.table.refresh();
            this.table.requestFocus();
        }
        
        super.setVisible( visible );


        if ( visible == true ) {
            this.table.requestFocus();

            

        }
       
        
    }

    
    /**
    Used to take the jDialog and center it within the given view.
    
    @author <A HREF="mailto:burton@relativity.yi.org">Kevin A. Burton</A>
    @version $Id: QuickBuffer.java,v 1.3 2000/08/08 00:05:51 burtonator Exp $
    */
    public void center() {

        if ( this.centered == false ) {
        
            if (this.getView() != null) {        
                int x = this.getView().getLocation().x + 
                        this.getView().getSize().width / 2 - 
                        this.getSize().width / 2;
            
                int y = this.getView().getLocation().y + 
                        this.getView().getSize().height / 2 - 
                        this.getSize().height / 2;
            
    
    
                jEdit.setProperty( "quickfile.open.x", Integer.toString(x) );
                jEdit.setProperty( "quickfile.open.y", Integer.toString(y) );
                jEdit.setProperty( "quickfile.open.width", Integer.toString( this.getSize().width ) );
                jEdit.setProperty( "quickfile.open.height", Integer.toString( this.getSize().height ) );
    
                GUIUtilities.loadGeometry( this, "quickfile.open" );
    
            }
            
            this.centered = true;
        }
    }
    
    public View getView() {
        return this.view;
    }

    public void actionPerformed(ActionEvent evt) {
        
        if ( evt.getSource() == this.cancel ) {
            this.setVisible(false);
        } else if (evt.getSource() == this.ok ) {
            this.selectCurrentFile();
        }
        
    }

    /**
    Get the current file from the JTable and set it in jEdit.

    @author <A HREF="mailto:burton@relativity.yi.org">Kevin A. Burton</A>
    @version $Id: QuickBuffer.java,v 1.3 2000/08/08 00:05:51 burtonator Exp $
    */
    public void selectCurrentFile() {

        if ( this.isVisible() ) {
            this.setVisible( false );
            this.getView().setBuffer( this.table.getSelectedFile() );
        }

    }
    
    //key press code for the jtable...
    public void keyPressed(KeyEvent event) {}
    public void keyReleased(KeyEvent event) {}

    /**
    @author <A HREF="mailto:burton@relativity.yi.org">Kevin A. Burton</A>
    @version $Id: QuickBuffer.java,v 1.3 2000/08/08 00:05:51 burtonator Exp $
    */
    public void keyTyped(KeyEvent event) {
        
        if ( event.getKeyChar() == KeyEvent.VK_ENTER ) {
            this.selectCurrentFile();
        } else if ( event.getKeyChar() == KeyEvent.VK_ESCAPE ) {
            this.setVisible(false);
        }



    }


    public void mousePressed(MouseEvent evt)  { }
    public void mouseReleased(MouseEvent evt) { }
    public void mouseEntered(MouseEvent evt)  { }
    public void mouseExited(MouseEvent evt)   { }

    /**
    MouseListener interface.

    Determines when the user clicks on the JTree.

    @author <A HREF="mailto:burton@relativity.yi.org">Kevin A. Burton</A>    
    */
    public void mouseClicked(MouseEvent evt) {

        if(evt.getClickCount() == 2) {
            this.selectCurrentFile();
        }

    }
    
}
